require 'remme/performer/base'

class Scheduler < Performer::Base
  class << self
    TIMEOUT_MULTIPLIER = 60

    attr_accessor :completed
    alias :completed? :completed

    @@class_instance_fields = %i(
      after_hook before_hook
      message notifier
    )

    def run(options)
      build_class_instance_fields(options)
      perform_in(options.delete(:timeout).to_i * TIMEOUT_MULTIPLIER)
    end

    @@class_instance_fields.each do |field|
      define_method(field) { instance_variable_get("@#{field}") }
    end

    def build_class_instance_fields(options)
      @@class_instance_fields.each do |field_name|
        instance_variable_set("@#{field_name}", options.delete(field_name))
      end
    end
  end

  def perform
    complete_after { notify }
  end

  private

  def notify
    with_callbacks do
      self.class.notifier.run(self.class.message)
    end
  end

  def complete_after
    yield
    self.class.completed = true
  end

  def with_callbacks
    self.class.before_hook.call
    yield
    self.class.after_hook.call
  end
end
